'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createSplashSubscriber = createSplashSubscriber;

var _electron = require('electron');

var _path = require('path');

var _fs = require('fs');

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _fs2 = require('../utils/fs');

var _launch = require('./launch');

var _menu = require('./menu');

var _prepareEnv = require('./prepare-env');

var _prepareEnv2 = _interopRequireDefault(_prepareEnv);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = require('electron-log');

const kernelspecs = require('kernelspecs');
const jupyterPaths = require('jupyter-paths');
const path = require('path');

const argv = require('yargs').version().usage('Usage: nteract <notebooks> [options]').example('nteract notebook1.ipynb notebook2.ipynb', 'Open notebooks').example('nteract --kernel javascript', 'Launch a kernel').describe('kernel', 'Launch a kernel').default('kernel', 'python3').alias('k', 'kernel').alias('v', 'version').alias('h', 'help').describe('verbose', 'Display debug information').help('help').parse(process.argv.slice(1));

log.info('args', argv);

const notebooks = argv._.filter(x => /(.ipynb)$/.test(x)).filter(x => (0, _fs.existsSync)((0, _path.resolve)(x)));

_electron.ipcMain.on('new-kernel', (event, newKernel) => {
  (0, _launch.launchNewNotebook)(newKernel);
});

_electron.ipcMain.on('open-notebook', (event, filename) => {
  (0, _launch.launch)((0, _path.resolve)(filename));
});

const electronReady$ = _Rx2.default.Observable.fromEvent(_electron.app, 'ready');

const fullAppReady$ = _Rx2.default.Observable.zip(electronReady$, _prepareEnv2.default).first();

const jupyterConfigDir = path.join(_electron.app.getPath('home'), '.jupyter');
const nteractConfigFilename = path.join(jupyterConfigDir, 'nteract.json');

const prepJupyterObservable = _prepareEnv2.default.mergeMap(() =>
// Create all the directories we need in parallel
_Rx2.default.Observable.forkJoin(
// Ensure the runtime Dir is setup for kernels
(0, _fs2.mkdirpObservable)(jupyterPaths.runtimeDir()),
// Ensure the config directory is all set up
(0, _fs2.mkdirpObservable)(jupyterConfigDir)))
// Set up our configuration file
.mergeMap(() => (0, _fs2.readFileObservable)(nteractConfigFilename).catch(err => {
  if (err.code === 'ENOENT') {
    return (0, _fs2.writeFileObservable)(nteractConfigFilename, JSON.stringify({
      theme: 'light'
    }));
  }
  throw err;
}));

const kernelSpecsPromise = prepJupyterObservable.toPromise().then(() => kernelspecs.findAll());

/**
 * Creates an Rx.Subscriber that will create a splash page onNext and close the
 * splash page onComplete.
 * @return {Rx.Subscriber} Splash Window subscriber
 */
function createSplashSubscriber() {
  let win;

  return _Rx2.default.Subscriber.create(() => {
    win = new _electron.BrowserWindow({
      width: 565,
      height: 233,
      useContentSize: true,
      title: 'loading',
      frame: false,
      show: false
    });

    const index = (0, _path.join)(__dirname, '..', '..', 'static', 'splash.html');
    win.loadURL(`file://${ index }`);
    win.once('ready-to-show', () => {
      win.show();
    });
  }, null, () => {
    // Close the splash page when completed
    if (win) {
      win.close();
    }
  });
}

const appAndKernelSpecsReady = _Rx2.default.Observable.zip(fullAppReady$, kernelSpecsPromise);

const splashSubscription = electronReady$
// TODO: Take until first window is shown
.takeUntil(appAndKernelSpecsReady).subscribe(createSplashSubscriber());

function closeAppOnNonDarwin() {
  // On macOS, we want to keep the app and menu bar active
  if (process.platform !== 'darwin') {
    _electron.app.quit();
  }
}
const windowAllClosed = _Rx2.default.Observable.fromEvent(_electron.app, 'window-all-closed');
const windowClosedSubscription = windowAllClosed.skipUntil(appAndKernelSpecsReady).subscribe(closeAppOnNonDarwin);

const openFile$ = _Rx2.default.Observable.fromEvent(_electron.app, 'open-file', (event, filename) => ({ event, filename }));

function openFileFromEvent({ event, filename }) {
  event.preventDefault();
  (0, _launch.launch)((0, _path.resolve)(filename));
}

// Since we can't launch until app is ready
// and macOS will send the open-file events early,
// buffer those that come early.
openFile$.buffer(fullAppReady$) // Form an array of open-file events from before app-ready
.first() // Should only be the first
.subscribe(buffer => {
  // Now we can choose whether to open the default notebook
  // based on if arguments went through argv or through open-file events
  if (notebooks.length <= 0 && buffer.length <= 0) {
    log.info('launching an empty notebook by default');
    kernelSpecsPromise.then(specs => {
      let kernel;

      if (argv.kernel in specs) {
        kernel = argv.kernel;
      } else if ('python2' in specs) {
        kernel = 'python2';
      } else {
        const specList = Object.keys(specs);
        specList.sort();
        kernel = specList[0];
      }

      (0, _launch.launchNewNotebook)(kernel);
    });
  } else {
    notebooks.forEach(f => {
      try {
        (0, _launch.launch)((0, _path.resolve)(f));
      } catch (e) {
        log.error(e);
        console.error(e);
      }
    });
  }
  buffer.forEach(openFileFromEvent);
});

// All open file events after app is ready
openFile$.skipUntil(fullAppReady$).subscribe(openFileFromEvent);

fullAppReady$.subscribe(() => {
  kernelSpecsPromise.then(kernelSpecs => {
    if (Object.keys(kernelSpecs).length !== 0) {
      // Get the default menu first
      _electron.Menu.setApplicationMenu(_menu.defaultMenu);
      // Let the kernels/languages come in after
      (0, _menu.loadFullMenu)().then(menu => _electron.Menu.setApplicationMenu(menu));
    } else {
      _electron.dialog.showMessageBox({
        type: 'warning',
        title: 'No Kernels Installed',
        buttons: [],
        message: 'No kernels are installed on your system.',
        detail: 'No kernels are installed on your system so you will not be ' + 'able to execute code cells in any language. You can read about ' + 'installing kernels at ' + 'https://ipython.readthedocs.io/en/latest/install/kernel_install.html'
      }, index => {
        if (index === 0) {
          _electron.app.quit();
        }
      });
    }
  }).catch(err => {
    _electron.dialog.showMessageBox({
      type: 'error',
      title: 'No Kernels Installed',
      buttons: [],
      message: 'No kernels are installed on your system.',
      detail: 'No kernels are installed on your system so you will not be ' + 'able to execute code cells in any language. You can read about ' + 'installing kernels at ' + 'https://ipython.readthedocs.io/en/latest/install/kernel_install.html' + `\nFull error: ${ err.message }`
    }, index => {
      if (index === 0) {
        _electron.app.quit();
      }
    });
  });
});