'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ipyKernelTryObservable = ipyKernelTryObservable;
exports.condaInfoObservable = condaInfoObservable;
exports.condaEnvsObservable = condaEnvsObservable;
exports.createKernelSpecsFromEnvs = createKernelSpecsFromEnvs;
exports.condaKernelsObservable = condaKernelsObservable;

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _spawnRx = require('spawn-rx');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const path = require('path');

/**
 * ipyKernelTryObservable checks for the existence of ipykernel in the environment.
 * @param  {Object} env - Current environment
 * @returns {Observable}  Source environment
 */
function ipyKernelTryObservable(env) {
  const executable = path.join(env.prefix, 'bin', 'python');
  return (0, _spawnRx.spawn)(executable, ['-m', 'ipykernel', '--version'], { split: true }).filter(x => x.source && x.source === 'stdout').mapTo(env).catch(err => _Rx2.default.Observable.empty());
}

/**
  * condaInfoObservable executes the conda info --json command and maps the
  * result to an observable that parses through the environmental informaiton.
  * @returns {Observable}  JSON parsed information
  */
function condaInfoObservable() {
  return (0, _spawnRx.spawn)('conda', ['info', '--json']).map(info => JSON.parse(info));
}

/**
  * condaEnvsObservable will return an observable that emits the environmental
  * paths of the passed in observable.
  * @param {Observable} condaInfo$ - Environmental information
  * @returns {Observable}  List of envionmental variables
  */
function condaEnvsObservable(condaInfo$) {
  return condaInfo$.map(info => {
    const envs = info.envs.map(env => ({ name: path.basename(env), prefix: env }));
    envs.push({ name: 'root', prefix: info.root_prefix });
    return envs;
  }).map(envs => envs.map(ipyKernelTryObservable)).mergeAll().mergeAll().toArray();
}

/**
  * createKernelSpecsFromEnvs generates a dictionary with the supported langauge
  * paths.
  * @param {Observable} envs - Environmental elements
  * @returns {Object}   Dictionary containing supported langauges paths.
  */
function createKernelSpecsFromEnvs(envs) {
  const displayPrefix = 'Python'; // Or R
  const languageKey = 'py'; // or r

  const languageExe = 'bin/python';
  const jupyterBin = 'bin/jupyter';

  const langEnvs = {};

  Object.keys(envs).forEach(env => {
    const base = env.prefix;
    const exePath = path.join(base, languageExe);
    const envName = env.name;
    const name = `conda-env-${ envName }-${ languageKey }`;
    langEnvs[name] = {
      display_name: `${ displayPrefix } [conda env:${ envName }]`,
      argv: [exePath, '-m', 'ipykernel', '-f', '{connection_file}'],
      language: 'python'
    };
  });
  return langEnvs;
}

/**
  * condaKernelObservable generates an observable containing the supported languages
  * environmental elements.
  * @returns {Observable}  Supported language elements
  */
function condaKernelsObservable() {
  return condaEnvsObservable(condaInfoObservable()).map(createKernelSpecsFromEnvs);
}